@@ -1,20 +1,20 @@
-# $Id: mvt.R 305 2015-07-21 11:12:52Z thothorn $
+# $Id: mvt.R 316 2015-12-21 08:35:05Z thothorn $
 
+##' Do we have a correlation matrix?
+##' @param x typically a matrix
 chkcorr <- function(x) {
-
-    if (!is.matrix(x)) return(FALSE)
+    if (!is.matrix(x) || (d <- dim(x))[1] != d[2])
+        return(FALSE)
     rownames(x) <- colnames(x) <- NULL
     storage.mode(x) <- "numeric"
     ONE <- 1 + sqrt(.Machine$double.eps)
 
-    ret <- (min(x) < -ONE || max(x) > ONE) ||
-           !isTRUE(all.equal(diag(x), rep(1, nrow(x))))
-    !ret
+    ## return
+    -ONE <= min(x) && max(x) <= ONE && isTRUE(all.equal(diag(x), rep(1, d[1])))
 }
 
 checkmvArgs <- function(lower, upper, mean, corr, sigma)
 {
-    UNI <- FALSE
     if (!is.numeric(lower) || !is.vector(lower))
         stop(sQuote("lower"), " is not a numeric vector")
     if (!is.numeric(upper) || !is.vector(upper))
@@ -25,7 +25,7 @@ checkmvArgs <- function(lower, upper, mean, corr, sigma)
         stop(sQuote("lower"), " not specified or contains NA")
     if (is.null(upper) || any(is.na(upper)))
         stop(sQuote("upper"), " not specified or contains NA")
-    rec <- cbind(lower, upper, mean)
+    rec <- cbind(lower, upper, mean)# <--> recycling to same length
     lower <- rec[,"lower"]
     upper <- rec[,"upper"]
     if (!all(lower <= upper))
@@ -44,6 +44,7 @@ checkmvArgs <- function(lower, upper, mean, corr, sigma)
         warning("both ", sQuote("corr"), " and ", sQuote("sigma"),
                 " specified: ignoring ", sQuote("sigma"))
     }
+    UNI <- FALSE
     if (!is.null(corr)) {
          if (!is.numeric(corr))
              stop(sQuote("corr"), " is not numeric")
@@ -199,15 +200,14 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
     addargs <- list(...)
     if (length(addargs) > 0)
         algorithm <- GenzBretz(...)
-    if (is.function(algorithm) || is.character(algorithm))
+    else if (is.function(algorithm) || is.character(algorithm))
         algorithm <- do.call(algorithm, list())
 
     ### handle cases where the support is the empty set
-    if (any(abs(lower - upper) < sqrt(.Machine$double.eps)) ||
-        any(is.na(lower - upper))) {
-        RET <- list(value = 0, error = 0, msg = "lower == upper")
-        return(RET)
-    }
+    ##  Note: checkmvArgs() has been called ==> lower, upper are *not* NA
+    if (any(abs(d <- lower - upper) < sqrt(.Machine$double.eps)*(abs(lower)+abs(upper)) |
+            lower == upper)) ## e.g. Inf == Inf
+	return(list(value = 0, error = 0, msg = "lower == upper"))
 
     n <- ncol(corr)
     if (is.null(n) || n < 2) stop("dimension less then n = 2")
@@ -227,8 +227,8 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
     ###         mean=c(0, 0, 0), sigma=S, algorithm = Miwa())
     ###         returned NA
 
-    if (class(algorithm) == "Miwa") {
-        if (any(infin == -1) & n >= 3) {
+    if (inherits(algorithm, "Miwa")) {
+        if (n >= 3 && any(infin == -1)) {
             WhereBothInfIs <- which(infin == -1)
             n <- n - length(WhereBothInfIs)
             corr <- corr[-WhereBothInfIs, -WhereBothInfIs]
@@ -236,11 +236,11 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
             lower <- lower[-WhereBothInfIs]
         }
 
-        if (any(infin == 0) & n >= 2) {
+        if (n >= 2 && any(infin == 0)) {
             WhereNegativInfIs <- which(infin==0)
             inversecorr <- rep(1, n)
             inversecorr[WhereNegativInfIs] <- -1
-            corr <- diag(inversecorr) %*% corr %*% diag(inversecorr)
+            corr <- diag(inversecorr) %*% corr %*% diag(inversecorr) ## MM_FIXME
             infin[WhereNegativInfIs] <- 1
 
             tempsaveupper <- upper[WhereNegativInfIs]
@@ -260,17 +260,16 @@ mvt <- function(lower, upper, df, corr, delta, algorithm = GenzBretz(), ...)
 
 
     ret <- probval(algorithm, n, df, lower, upper, infin, corr, corrF, delta)
-    error <- ret$error; value <- ret$value; inform <- ret$inform
+    inform <- ret$inform
+    msg <-
+        if (inform == 0) "Normal Completion"
+    else if (inform == 1) "Completion with error > abseps"
+    else if (inform == 2) "N greater 1000 or N < 1"
+    else if (inform == 3) "Covariance matrix not positive semidefinite"
+    else inform
 
-    msg <- NULL
-    if (inform == 0) msg <- "Normal Completion"
-    if (inform == 1) msg <- "Completion with error > abseps"
-    if (inform == 2) msg <- "N greater 1000 or N < 1"
-    if (inform == 3) msg <- "Covariance matrix not positive semidefinite"
-    if (is.null(msg)) msg <- inform
-
-    RET <- list(value = value, error = error, msg = msg)
-    return(RET)
+    ## return including error est. and msg:
+    list(value = ret$value, error = ret$error, msg = msg)
 }
 
 rmvt <- function(n, sigma = diag(2), df = 1,
@@ -337,9 +336,10 @@ dmvt <- function(x, delta = rep(0, p), sigma = diag(p), df = 1,
 qmvnorm <- function(p, interval = NULL,
                     tail = c("lower.tail", "upper.tail", "both.tails"),
                     mean = 0, corr = NULL, sigma = NULL, algorithm =
-                    GenzBretz(), ...)
+                    GenzBretz(),
+                    ptol = 0.001, maxiter = 500, trace = FALSE, ...)
 {
-    if (length(p) != 1 || (p <= 0 || p >= 1))
+    if (length(p) != 1 || p < 0 || p > 1)
         stop(sQuote("p"), " is not a double between zero and one")
 
     dots <- dots2GenzBretz(...)
@@ -360,11 +360,11 @@ qmvnorm <- function(p, interval = NULL,
         if (tail == "both.tails") p <- ifelse(p < 0.5, p / 2, 1 - (1 - p)/2)
         q <- qnorm(p, mean = args$mean, sd = args$sigma,
                    lower.tail = (tail != "upper.tail"))
-        return( list(quantile = q, f.quantile = 0) )
+        return( list(quantile = q, f.quantile = p) )
     }
     dim <- length(args$mean)
 
-    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) 
+    if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE))
         runif(1)
     R.seed <- get(".Random.seed", envir = .GlobalEnv)
 
@@ -383,19 +383,24 @@ qmvnorm <- function(p, interval = NULL,
            },)
            ret <- pmvnorm(lower = low, upper = upp, mean = args$mean,
                           corr = args$corr, sigma = args$sigma,
-                          algorithm = algorithm) - p
-           return(ret^2)
+                          algorithm = algorithm)
+           if(tail == "upper.tail") ## get_quant_loclin assumes an increasing function
+             ret <- 1-ret
+           return(ret)
     }
-
-    pstart <- switch(tail, "both.tails" = (1 - (1 - p)/2)^(1/dim),
-                           "upper.tail" = 1 - p^(1/dim),
-                           "lower.tail" = p^(1/dim))
-    qroot <- optim(qnorm(pstart), pfct, method = "BFGS", control = list(maxit = 1000))
-
-    if (qroot$convergence != 0)
-        stop("Search for quantile terminated unsuccessfully:", qroot$message)
-
-    qroot <- list(quantile = qroot$par, f.quantile = qroot$value)
+    if(is.null(interval)){
+      intp <- switch(tail,
+                     both.tails = (1 - (1 - p)/2)^(c(1,1/dim)),
+                     upper.tail = c(0.5,0.9), ## not so smart
+                     lower.tail = p^c(1,1/dim))
+      interval <- qnorm(intp)*c(0.8, 1.25)
+    } ## note: interval does not need to cover the true root for get_quant_loclin
+    if(tail == "upper.tail") ## get_quant_loclin assumes an increasing function
+      p <- 1-p
+    qroot <- get_quant_loclin(pfct, p, interval=interval,
+                              link="probit",
+                              ytol=ptol, maxiter=maxiter, verbose=trace)
+    qroot$f.quantile <- qroot$f.quantile - p
     qroot
 }
 
@@ -403,7 +408,8 @@ qmvt <- function(p, interval = NULL,
                  tail = c("lower.tail", "upper.tail", "both.tails"),
                  df = 1, delta = 0, corr = NULL, sigma = NULL,
                  algorithm = GenzBretz(),
-                 type = c("Kshirsagar", "shifted"), ...) {
+                 type = c("Kshirsagar", "shifted"),
+                 ptol = 0.001, maxiter = 500, trace = FALSE, ...) {
 
     if (length(p) != 1 || (p <= 0 || p >= 1))
         stop(sQuote("p"), " is not a double between zero and one")
@@ -428,7 +434,7 @@ qmvt <- function(p, interval = NULL,
         } else {
             q <- qt(p, df = df, ncp = args$mean, lower.tail = (tail != "upper.tail"))
         }
-        qroot <- list(quantile = q, f.quantile = 0)
+        qroot <- list(quantile = q, f.quantile = p)
         return(qroot)
     }
 
@@ -453,30 +459,41 @@ qmvt <- function(p, interval = NULL,
            },)
            ret <- pmvt(lower = low, upper = upp, df = df, delta = args$mean,
                        corr = args$corr, sigma = args$sigma,
-                       algorithm = algorithm, type = type) - p
-           return(ret^2)
+                       algorithm = algorithm, type = type)
+        if(tail == "upper.tail") ## get_quant_loclin assumes an increasing function
+          ret <- 1-ret
+        return(ret)
     }
-
-    pstart <- switch(tail, "both.tails" = (1 - (1 - p)/2)^(1/dim),          
-                           "upper.tail" = 1 - p^(1/dim),
-                           "lower.tail" = p^(1/dim))    
-    par <- ifelse(is.finite(df) && (df > 0), qt(pstart, df = df), qnorm(pstart))
-    qroot <- optim(par, pfct, method = "BFGS", control = list(maxit = 1000))
-
-    if (qroot$convergence != 0)
-        stop("Search for quantile terminated unsuccessfully:", qroot$message)
-
-    qroot <- list(quantile = qroot$par, f.quantile = qroot$value)
+    if(is.null(interval)){
+      intp <- switch(tail,
+                     both.tails = (1 - (1 - p)/2)^(c(1,1/dim)),
+                     upper.tail = c(0.5,0.9), ## not so smart
+                     lower.tail = p^c(1,1/dim))
+      if(is.finite(df) && (df > 0)){
+        intq <- qt(intp, df = df)
+      } else {
+        intq <- qnorm(intp)
+      }
+      interval <- intq*c(0.8, 1.25)
+    } ## note: interval does not need to cover the true root for get_quant_loclin, just initial guesses
+    if(tail == "upper.tail") ## get_quant_loclin assumes an increasing function
+      p <- 1-p
+    link <- ifelse(df <= 7 & df > 0, "cauchit", "probit")
+    qroot <- get_quant_loclin(pfct, p, interval=interval,
+                              link=link,
+                              ytol=ptol, maxiter=maxiter, verbose=trace)
+    qroot$f.quantile <- qroot$f.quantile - p
     qroot
-
 }
 
+
 GenzBretz <- function(maxpts = 25000, abseps = 0.001, releps = 0) {
     structure(list(maxpts = maxpts, abseps = abseps, releps = releps),
               class = "GenzBretz")
 }
 
 Miwa <- function(steps = 128) {
+    if (steps > 4098) stop("maximum number of steps is 4098")
     structure(list(steps = steps), class = "Miwa")
 }
 
@@ -491,21 +508,21 @@ probval.GenzBretz <- function(x, n, df, lower, upper, infin, corr, corrF, delta)
     upper[ isInf(upper)] <- 0
 
     error <- 0; value <- 0; inform <- 0
-    ret <- .C("C_mvtdst", N = as.integer(n),
-                          NU = as.integer(df),
-                          LOWER = as.double(lower),
-                          UPPER = as.double(upper),
-                          INFIN = as.integer(infin),
-                          CORREL = as.double(corrF),
-                          DELTA = as.double(delta),
-                          MAXPTS = as.integer(x$maxpts),
-                          ABSEPS = as.double(x$abseps),
-                          RELEPS = as.double(x$releps),
-                          error = as.double(error),
-                          value = as.double(value),
-                          inform = as.integer(inform),
-                          RND = as.integer(1)) ### init RNG
-    ret
+    .C("C_mvtdst",
+       N = as.integer(n),
+       NU = as.integer(df),
+       LOWER = as.double(lower),
+       UPPER = as.double(upper),
+       INFIN = as.integer(infin),
+       CORREL = as.double(corrF),
+       DELTA = as.double(delta),
+       MAXPTS = as.integer(x$maxpts),
+       ABSEPS = as.double(x$abseps),
+       RELEPS = as.double(x$releps),
+       error = as.double(error),
+       value = as.double(value),
+       inform = as.integer(inform),
+       RND = as.integer(1)) ### init RNG
 }
 
 probval.Miwa <- function(x, n, df, lower, upper, infin, corr, corrF, delta) {
@@ -514,7 +531,7 @@ probval.Miwa <- function(x, n, df, lower, upper, infin, corr, corrF, delta) {
         stop("Miwa algorithm cannot compute t-probabilities")
 
     if (n > 20)
-        stop("Miwa algorithm cannot compute exact probabilities for n > 20")
+        stop("Miwa algorithm cannot compute probabilities for dimension n > 20")
 
     sc <- try(solve(corr))
     if (inherits(sc, "try-error"))
